# -*- coding: utf-8 -*-

# =============
#  - Imports -
# =============

# --------------------
#  - Python modules -
# --------------------

from configobj import ConfigObj
from random import choice as rChoice

# --------------------------
#  - EventScripts modules -
# --------------------------

import es
import gamethread
from playerlib import getPlayer
from popuplib import easymenu

# -------------------------
#  - Third Party modules -
# -------------------------

from spe.games import cstrike

# ==========================
#  - ES Hidden code start -
# ==========================

# ----------------------
#  - Global variables -
# ----------------------

players = {}

# -------------
#  - Classes -
# -------------

class Config(object):
    def __init__(self):
        self.config  = ConfigObj(es.getAddonPath("hidden") + "/hidden_config.ini")
        self.general = self.config["general"]
        self.hidden  = self.config["hidden"]
        self.iris    = self.config["iris"]
    
    def getDarkMode(self):
        value = int(self.general["dark_mode"])
        if not value:
            return "g"
        elif value == 1:
            return "b"
        elif value == 2:
            return "a"
    
    def getHiddenHealth(self):
        return int(self.hidden["health"])
    
    def getHiddenSpeed(self):
        return float(self.hidden["speed"])
    
    def getHiddenAdditionalHealth(self):
        return int(self.hidden["additional_health"])
    
    def getHiddenExtraSpeed(self):
        return float(self.hidden["extra_speed"])
    
    def getHiddenCount(self):
        return int(self.hidden["count"])
    
    def getIrisBotPrimary(self):
        return self.iris["bot_primary"]
    
    def getIrisBotSecondary(self):
        return self.iris["bot_secondary"]
    
    def getIrisMenuTime(self):
        return int(self.iris["menu_time"])

config = Config()

class Menus(object):
    def __init__(self):
        self.main      = easymenu("hidden_main", "_popup_choice", self._mainSelect)
        self.secondary = easymenu("hidden_secondary", "_popup_choice", self._secondarySelect)
        self.support   = easymenu("hidden_support", "_popup_choice", self._supportSelect)
        self.assault   = easymenu("hidden_assault", "_popup_choice", self._assaultSelect)
        self.addition  = easymenu("hidden_addition", "_popup_choice", self._additionSelect)
    
    def sendMain(self, userid):
        self.main.send(userid)
    
    def sendSecondary(self, userid):
        self.secondary.send(userid)
    
    def sendSupport(self, userid):
        self.support.send(userid)
    
    def sendAssault(self, userid):
        self.assault.send(userid)
    
    def sendAddition(self, userid):
        self.addition.send(userid)
    
    def _mainSelect(self, userid, choice, popupid):
        players[str(userid)].mainSelect(choice)
    
    def _secondarySelect(self, userid, choice, popupid):
        players[str(userid)].secondarySelect(choice)
    
    def _supportSelect(self, userid, choice, popupid):
        players[str(userid)].primarySelect(choice)
    
    def _assaultSelect(self, userid, choice, popupid):
        players[str(userid)].primarySelect(choice)
    
    def _additionSelect(self, userid, choice, popupid):
        players[str(userid)].additionSelect(choice)
    
    def scriptLoad(self):
        self.main.settitle("[Hidden] Assault or Support?")
        self.main.addoption(1, "[√] Assault")
        self.main.addoption(2, "[√] Support")
        
        self.secondary.settitle("[Hidden] Secondary")
        self.secondary.addoption("fiveseven", "[√] FN FiveSeven Pistol (ES FiveSeven)")
        self.secondary.addoption("p228", "[√] FNP-9 Pistol (P228)")
        
        self.support.settitle("[Hidden] [√] Support")
        self.support.addoption("mp5navy", "[√] FN Navy 3200 Sub Machine Gun  (MP5Navy)")
        self.support.addoption("tmp", "[√] FN Tactial Machine Gun  (TMP)")
        self.support.addoption("xm1014", "[√] FN303 Less Lethal Launcher (XM1014)")
        
        self.assault.settitle("[Hidden] [√] Assault")
        self.assault.setdescription("M4A1 Carbine will set your HP to 50.")
        self.assault.addoption("m4a1", "[√] FN4700 Assault Rifle (M4A1 Carbine)")
        self.assault.addoption("famas", "[√] FN2000 Assault Rifle (FAMAS)")
        self.assault.addoption("m3", "[√] Remington 870 MCS Shotgun (M3)")
        self.assault.addoption("xm1014", "[√] FN303 Less Lethal Launcher (XM1014)")
        
        self.addition.settitle("[Hidden] Choose an Additional")
        self.addition.addoption(1, "[√] Light Amplification Goggles (NVGs)")
        self.addition.addoption(2, "[√] High-Explosive Grenade")
        self.addition.addoption(3, "[√] Flashbang Grenade")
    
    def scriptUnload(self):
        self.main.delete()
        self.secondary.delete()
        self.support.delete()
        self.assault.delete()
        self.addition.delete()

menus = Menus()

class Sounds(object):
    def __init__(self):
        self.taunt = ("radiotaunts08", "radiotaunts02", "radiotaunts04")
        self.start = ("roundstart02", "roundstart04", "roundstart05")
        self.end   = ("roundend01", "roundend06", "roundend07")
        self.scary = {"hidden": ("imhere01", "overhere02", "turnaround02"), "iris": ("ragealert02", "ragealert04", "ragealert05")}
        self.death = {"hidden": ("death01", "death05"), "iris": ("death14", "death04")}
    
    def getRandomStart(self):
        return rChoice(self.start)
    
    def getRandomScary(self, key):
        return rChoice(self.scary[key])
    
    def getRandomDeath(self, key):
        return rChoice(self.death[key])
    
    def getRandomEnd(self):
        return rChoice(self.end)
    
    def getRandomTaunt(self):
        return rChoice(self.taunt)
    
    def getMediKitPath(self):
        return "items/smallmedkit1.wav"
    
    def addDownloads(self):
        downloads = []
        for sound in self.taunt:
            downloads.append("IRIS-617%s.mp3" % sound)
            downloads.append("617-%s.mp3" % sound)
        for sound in self.start:
            downloads.append("IRIS-%s.wav" % sound)
        for sound in self.end:
            downloads.append("IRIS-%s.wav" % sound)
        for sound in self.scary["hidden"]:
            downloads.append("617-%s.mp3" % sound)
        for sound in self.scary["iris"]:
            downloads.append("IRIS-%s.wav" % sound)
        for sound in self.death["hidden"]:
            downloads.append("617-%s.mp3" % sound)
        for sound in self.death["iris"]:
            downloads.append("IRIS-%s.wav" % sound)
        for sound in downloads:
            es.stringtable("downloadables", "sound/hidden/%s" % sound)
    
    def startScaryLoop(self):
        gamethread.delayedname(30, "hidden_scary", self.scaryLoop)
    
    def scaryLoop(self):
        if es.getplayercount(2) and es.getplayercount(3):
            es.cexec_all("play", "hidden/617-%s.mp3" % self.getRandomScary("hidden"))
            gamethread.delayed(4, es.cexec_all, ("play", "hidden/IRIS-%s.wav" % self.getRandomScary("iris")))
        gamethread.delayedname(30, "hidden_scary", self.scaryLoop)
    
    def stopScaryLoop(self):
        gamethread.cancelDelayed("hidden_scary")

sounds = Sounds()

class Model(object):
    def __init__(self):
        self.model = "models/player/elis/hdv3/hidden.mdl"
    
    def addDownloads(self):
        es.stringtable("downloadables", "models/player/elis/hdv3/hidden.dx80.vtx")
        es.stringtable("downloadables", "models/player/elis/hdv3/hidden.dx90.vtx")
        es.stringtable("downloadables", "models/player/elis/hdv3/hidden.mdl")
        es.stringtable("downloadables", "models/player/elis/hdv3/hidden.phy")
        es.stringtable("downloadables", "models/player/elis/hdv3/hidden.sw.vtx")
        es.stringtable("downloadables", "models/player/elis/hdv3/hidden.vvd")
        es.stringtable("downloadables", "materials/models/player/elis/hd/Hidden_head.vmt")
        es.stringtable("downloadables", "materials/models/player/elis/hd/Hidden_Head.vtf")
        es.stringtable("downloadables", "materials/models/player/elis/hd/Hidden_head_normal.vtf")
        es.stringtable("downloadables", "materials/models/player/elis/hd/Hidden_Torso.vmt")
        es.stringtable("downloadables", "materials/models/player/elis/hd/Hidden_Torso.vtf")
        es.stringtable("downloadables", "materials/models/player/elis/hd/Hidden_torso_normal.vtf")
    
    def getModel(self):
        return self.model

model = Model()

class Hidden(object):
    class Player(object):
        IRIS_ASSAULT       = 1
        IRIS_ASSAULT_M4A1  = 3
        
        IRIS_ADDITION_NVGS = 1
        IRIS_ADDITION_HE   = 2
        
        def __init__(self, userid):
            self.menu       = None
            self.primary    = None
            self.secondary  = None
            self.addition   = None
            self.roundkills = 0
            self.menuUsed   = False
            self.userid     = userid
            self.player     = getPlayer(userid)
        
        def strip(self, knife=True, giveWeapons=False):
            self._fire("player_weaponstrip", "Kill")
            self._give("player_weaponstrip")
            self._fire("player_weaponstrip", "Strip")
            gamethread.queue(self._give, "item_assaultsuit")
            if knife:
                gamethread.queue(self._give, "weapon_knife")
            if giveWeapons:
                gamethread.delayed(.2, self._giveWeapons)
        
        def setFunctions(self, enable):
            if enable:
                self._fire("func_buyzone", "Enable")
                self._fire("func_bomb_target", "Enable")
                self._fire("func_hostage_rescue", "Enable")
            else:
                self._fire("func_buyzone", "Disable")
                self._fire("func_bomb_target", "Disable")
                self._fire("func_hostage_rescue", "Disable")
        
        def tryTeamChange(self, team):
            if team in (2, 3):
                hiddenCount = config.getHiddenCount()
                if hiddenCount == 1:
                    text = "One #greenHidden"
                elif hiddenCount > 1:
                    text = "%s #greenHiddens" % hiddenCount
                if team == 2 and es.getplayercount(2) > hiddenCount:
                    es.tell(self.userid, "#multi", "#green[#lightgreenES Hidden#green] #defaultThere can only be #lightgreen%s" % text)
                    return False
                if team == 3 and es.getplayercount(2) < hiddenCount:
                    es.tell(self.userid, "#multi", "#green[#lightgreenES Hidden#green] #defaultThere must be #lightgreen%s" % text)
                    return False
            return True
        
        def roundFreezeEnd(self):
            self.player.refreshAttributes()
            if self.player.teamid == 2:
                self._tellHidden()
            elif self.player.teamid == 3:
                self._tellIris()
            gamethread.delayed(5, es.cexec_all, ("play", "hidden/IRIS-%s" % sounds.getRandomStart()))
        
        def roundEnd(self):
            self.player.godmode(1)
            self.strip(knife=False)
        
        def playerSpawn(self):
            self.player.refreshAttributes()
            self.player.godmode(0)
            self.setFunctions(False)
            if self.player.teamid == 2:
                hiddenHealth = config.getHiddenHealth()
                self.strip()
                self.player.setSpeed(config.getHiddenSpeed())
                self.player.setModel(model.getModel())
                self.player.setColor(255, 255, 255, 25)
                self.player.setWeaponColor(255, 255, 255, 25)
                if self.roundkills:
                    additionalHealth = config.getHiddenAdditionalHealth()
                    if self.roundkills > 1:
                        text = "%s #greenIRIS Forces" % self.roundkills
                        hiddenHealth += (additionalHealth * self.roundkills)
                    else:
                        text = "One #greenIRIS Force"
                        hiddenHealth += additionalHealth
                    es.tell(self.userid, "#multi", "#green[#lightgreenES Hidden#green] #defaultYou get #green%i #lightgreenHP #defaultfor killing #lightgreen%s" % (hiddenHealth, text))
                    self.roundkills = 0
                self.player.setHealth(hiddenHealth)
            else:
                self.player.setColor(255, 255, 255, 255)
                self.player.setWeaponColor(255, 255, 255, 255)
                if self.player.steamid == "BOT":
                    gamethread.queue(self._give, "weapon_%s" % rChoice(config.getIrisBotSecondary()))
                    gamethread.queue(self._give, "weapon_%s" % rChoice(config.getIrisBotPrimary()))
                else:
                    es.tell(self.userid, "#multi", "#green[#lightgreenES Hidden#green] #defaultType #lightgreen!class #defaultto change your class.")
                    self.menuUsed = False
                    if self.menu is not None:
                        self.strip(giveWeapons=True)
                    else:
                        menus.sendMain(self.userid)
        
        def playerDeath(self, victim):
            victim.refreshAttributes()
            self.player.refreshAttributes()
            if self.player.teamid == 3 and victim.teamid == 2:
                hiddenCount = config.getHiddenCount()
                es.cexec(self.userid, "hidden/617-%s.mp3" % sounds.getRandomDeath("hidden"))
                if hiddenCount == 1:
                    text = "the"
                else:
                    text = "a"
                es.msg("#multi", "#green[#lightgreenES Hidden#green] %s #defaultis now %s #greenHidden" % (self.player.name, text))
                cstrike.switchTeam(self.userid, 2)
                cstrike.switchTeam(victim.userid, 3)
            else:
                additionalHealth = config.getHiddenAdditionalHealth()
                self.roundkills += 1
                self.player.setSpeed(self.player.getSpeed() + config.getHiddenExtraSpeed())
                self.player.health += additionalHealth
                es.cexec(self.userid, "playgamesound", "items/smallmedkit1.wav")
                es.cexec(victim.userid, "playgamesound", "hidden/IRIS-%s.wav" % sounds.getRandomDeath("iris"))
                es.msg("#multi", "#green[#lightgreenES Hidden#green] Hidden #lightgreen%s #defaultgot #green%i #lightgreenHP #defaultfor killing #green%s" % (self.player.name, additionalHealth, victim.name))
                gamethread.delayed(1, es.cexec_all, ("play", "hidden/%s%s.mp3" % (rChoice(('IRIS-617', '617-')), sounds.getRandomTaunt())))
            gamethread.queue(es.remove, "cs_ragdoll")
        
        def itemPickup(self, item):
            self.player.refreshAttributes()
            if self.player.teamid == 2 and item != "knife":
                self.strip()
        
        def sayCommand(self):
            self.player.refreshAttributes()
            if self.player.teamid == 3:
                menuTime = config.getIrisMenuTime()
                if not self.menuUsed:
                    self.menuUsed = True
                    menus.sendMain(self.userid)
                    gamethread.delayed(menuTime, self._setMenuUsed)
                else:
                    es.tell(self.userid, "#multi", "#green[#lightgreenES Hidden#green] #defaultYou can only change your class within #lightgreen%s #defaultseconds!" % menuTime)
            else:
                es.tell(self.userid, "#multi", "#green[#lightgreenES Hidden#green] Hidden #lightgreencan't change its class!")
        
        def mainSelect(self, choice):
            self.menu = choice
            if choice == self.IRIS_ASSAULT:
                menus.sendAssault(self.userid)
            else:
                menus.sendSupport(self.userid)
        
        def secondarySelect(self, choice):
            self.secondary = "weapon_%s" % choice
            self.strip(giveWeapons=True)
        
        def primarySelect(self, choice):
            self.primary = "weapon_%s" % choice
            menus.sendAddition(self.userid)
        
        def additionSelect(self, choice):
            self.addition = choice
            menus.sendSecondary(self.userid)
        
        def _giveWeapons(self):
            if self.menu == self.IRIS_ASSAULT:
                if self.menu == self.IRIS_ASSAULT_M4A1:
                    health = 50
                else:
                    health = 75
            else:
                health = 125
            self.player.setHealth(health)
            self._give(self.secondary)
            self._give(self.primary)
            gamethread.queue(self._handleAdditions)
        
        def _handleAdditions(self):
            if self.addition == self.IRIS_ADDITION_NVGS:
                self.player.setNightvision(1)
                if config.getDarkMode() == "a":
                    self.player.setNightvisionOn(1)
                es.tell(self.userid, "#multi", "#green[#lightgreenHidden#green] #defaultPress #greenN #defaultto de-/activate the #lightgreenLight Amplification Goggles (NVGs)")
            else:
                if self.addition == self.IRIS_ADDITION_HE:
                    self._give("weapon_hegrenade")
                else:
                    self._give("weapon_flashbang")
        
        def _setMenuUsed(self):
            self.menuUsed = False
        
        def _fire(self, *command):
            es.fire(self.userid, *command)
        
        def _give(self, command):
            es.give(self.userid, command)
        
        def _tellIris(self):
            es.centertell(self.userid, "617 has been spotted in this sector - MOVE OUT!")
            gamethread.delayed(1, es.centertell, (self.userid, "617 has been spotted in this sector - MOVE OUT!"))
            gamethread.delayed(2, es.centertell, (self.userid, "617 has been spotted in this sector - MOVE OUT!"))
        
        def _tellHidden(self):
            es.centertell(self.userid, "You are the Hidden - Rescue yourself!")
            gamethread.delayed(1, es.centertell, (self.userid, "You are the Hidden - Rescue yourself!"))
            gamethread.delayed(2, es.centertell, (self.userid, "You are the Hidden - Rescue yourself!"))
    
    def insertUserid(self, userid):
        players[str(userid)] = self.Player(userid)
    
    def removeUserid(self, userid=0, clear=False):
        if not clear:
            self._removeUserid(userid)
        elif players:
            for userid in players.keys():
                self._removeUserid(userid)
    
    def removeIdleWeapons(self):
        entlist = es.createentitylist()
        for index in entlist:
            if entlist[index]["classname"].startswith("weapon_") and es.getindexprop(index, "CBaseEntity.m_hOwnerEntity") == -1:
                es.remove(index)
    
    def mapStart(self):
        sounds.addDownloads()
        model.addDownloads()
        sounds.stopScaryLoop()
        self._setDarkMode()
    
    def roundStart(self):
        self._setDarkMode()
        es.msg("#multi", "#green[#lightgreenES Hidden#green] #lightgreen- #greenMISSION EXECUTE #lightgreen-")
    
    def roundFreezeEnd(self):
        self.removeIdleWeapons()
        sounds.startScaryLoop()
        for userid in players:
            players[userid].roundFreezeEnd()
    
    def roundEnd(self, winner):
        sounds.stopScaryLoop()
        if winner in (2, 3):
            if winner == 2:
                text = "WINS"
            else:
                text = "NEUTRALIZED"
                gamethread.delayed(.5, es.cexec_all, ('play', 'hidden/IRIS-%s.wav' % sounds.getRandomEnd()))
            es.centermsg("Subject 617")
            gamethread.delayed(1.3, es.centermsg, text)
        for userid in players:
            players[userid].roundEnd()
    
    def playerSpawn(self, userid):
        if userid in players:
            players[userid].playerSpawn()
    
    def playerDeath(self, attacker, victim):
        if int(attacker) and int(victim) and attacker != victim:
            players[attacker].playerDeath(getPlayer(victim))
    
    def itemPickup(self, userid, item):
        if userid in players:
            players[userid].itemPickup(item)
    
    def sayCommand(self, userid):
        if userid in players:
            players[userid].sayCommand()
    
    def scriptLoad(self):
        self._setDarkMode()
        for userid in es.getUseridList():
            self.insertUserid(userid)
        if not es.exists("saycommand", "!class"):
            es.regsaycmd("!class", "hidden/saycommand")
        es.addons.registerClientCommandFilter(self._clientCommandFilter)
        es.ServerVar("hidden_ver", 5).makepublic()
        menus.scriptLoad()
        sounds.addDownloads()
        model.addDownloads()
    
    def scriptUnload(self):
        self.removeUserid(clear=True)
        sounds.stopScaryLoop()
        menus.scriptUnload()
        if es.exists("saycommand", "!class"):
            es.unregsaycmd("!class")
        es.addons.unregisterClientCommandFilter(self._clientCommandFilter)
    
    def _clientCommandFilter(self, userid, args):
        if args[0] == "jointeam":
            return players[str(userid)].tryTeamChange(int(args[1]))
        return True
    
    def _setDarkMode(self):
        es.lightstyle(0, config.getDarkMode())
    
    def _removeUserid(self, userid):
        players[userid].setFunctions(True)
        del players[userid]

hidden = Hidden()

# ----------------
#  - Map events -
# ----------------

def es_map_start(ev):
    hidden.mapStart()

# ------------------
#  - Round events -
# ------------------

def round_freeze_end(ev):
    hidden.roundFreezeEnd()

def round_start(ev):
    hidden.roundStart()

def round_end(ev):
    hidden.roundEnd(int(ev["winner"]))

# -------------------
#  - Player events -
# -------------------

def player_activate(ev):
    hidden.insertUserid(ev["userid"])

def player_spawn(ev):
    if int(ev["es_userteam"]) > 1:
        hidden.playerSpawn(ev["userid"])

def player_death(ev):
    hidden.playerDeath(ev["attacker"], ev["userid"])

def player_disconnect(ev):
    hidden.removeUserid(ev["userid"])

# -----------------
#  - Item events -
# -----------------

def item_pickup(ev):
    hidden.itemPickup(ev["userid"], ev["item"])

# ------------------
#  - Say Commands -
# ------------------

def saycommand():
    hidden.sayCommand(str(es.getcmduserid()))

# -------------------------------
#  - EventScripts main functions
# -------------------------------

def load():
    hidden.scriptLoad()

def unload():
    hidden.scriptUnload()

# ========================
#  - ES Hidden code end -
# ========================
